-- GameKit Sample Chat Project

-- Usage
--   Make sure you are authenticated under the Game Center section of your device's settings.
--   Have two different devices each with a different Game Center account connected.
--   The two accounts must be added as friends in Game Center.
--   On one of the devices, click Create Match and invite the other account.
--   If successful, you should have a textbox and a button to send messages to the other player.

--- localPlayer: objc.GKLocalPlayer
--- viewController: objc.UIViewController
--- matchmakerViewController: objc.GKMatchmakerViewController

-- Use this function to perform your initial setup
function setup()
    viewController = objc.viewer
    
    localPlayer = objc.GKLocalPlayer.localPlayer
        
    matchmakerViewControllerDelegate = MatchmakerViewControllerDelegateType()
    localPlayerListener = LocalPlayerListenerType()
    matchDelegate = MatchDelegateType()

    if not localPlayer.authenticated then
        -- Setting the authenticateHandler will generally trigger authentication.
        -- If not, check that you are properly connected under Game Center in your device's settings.
        localPlayer.authenticateHandler = authenticateHandler
    end

    localPlayer:registerListener_(localPlayerListener)

    createStartButtons()
end

function createStartButtons()
    -- There is currently an issue in Codea where this doesn't clear parameters properly
    -- and the new parameters end-up overlapping with the old ones.
    parameter.clear()

    if localPlayer.authenticated then
        parameter.action("Create Match", createMatch)
    end
end

function createMatchButtons()
    -- There is currently an issue in Codea where this doesn't clear parameters properly
    -- and the new parameters end-up overlapping with the old ones.
    parameter.clear()

    parameter.action("Disconnect", endMatch)
    parameter.text("Message")
    parameter.action("Send", sendMessage)
end

function willClose()
    endMatch()
end

function draw()
    background(40, 40, 50)

    strokeWidth(5)

    text("Authenticated: " .. (localPlayer.authenticated and "true" or "false"), WIDTH / 2, HEIGHT - 100)
    text("Alias: " .. localPlayer.alias, WIDTH / 2, HEIGHT - 120)
    text("Match: " .. (currentMatch ~= nil and "Yes" or "No"), WIDTH / 2, HEIGHT - 140)
end
