MatchDelegateType = objc.delegate("GKMatchDelegate")

-- This is called by GameKit when a player connection state changes (e.g. he disconnected)
function MatchDelegateType:match_player_didChangeConnectionState_(objMatch, objPlayer, intState)
    if intState == objc.enum.GKPlayerConnectionState.GKPlayerStateDisconnected then
        print(objPlayer.alias .. " disconnected")
        endMatch()
    end
end

-- This is called by GameKit when receiving a message from another player.
-- Messages are sent as NSData and must be converted to a string.
function MatchDelegateType:match_didReceiveData_fromRemotePlayer_(objMatch, objData, objPlayer)
    local string = objc.NSString:stringWithUTF8String_(objData.bytes)
    print(objPlayer.alias .. ": " .. string)
end

function createMatch()
    if not localPlayer.authenticated then return end
    
    request = objc.GKMatchRequest()
    request.minPlayers = 2
    request.maxPlayers = 2

    -- "Unique" number so we only find matches for this sample project
    request.playerGroup = 1911316125
    
    matchmakerViewController = objc.GKMatchmakerViewController:alloc():initWithMatchRequest_(request)
    matchmakerViewController.matchmakerDelegate = matchmakerViewControllerDelegate

    -- This is crashing since iOS 16.4 ("known" issue)
    --matchmakerViewController.matchmakingMode = objc.enum.GKMatchmakingMode.inviteOnly
        
    viewController:presentModalViewController_animated_(matchmakerViewController, true)
end

function sendMessage()
    if currentMatch ~= nil then
        -- Messages must be sent as NSData. The encoding enum is not currently exposed under objc.enum.
        local data = objc.string(Message):dataUsingEncoding_(4) --utf8
        currentMatch:sendDataToAllPlayers_withDataMode_error_(data, objc.enum.GKMatchSendDataMode.GKMatchSendDataReliable, nil)
        print(localPlayer.alias .. ": " .. Message)
    end

    Message = ""

    createMatchButtons()
end

function endMatch()
    if currentMatch ~= nil then
        currentMatch:disconnect()
        currentMatch = nil
        print("Match ended")
    end

    createStartButtons()
end
